@@ -43,7 +43,7 @@
      #<DateTime 1986-10-21T22:00:00.000-02:00>
 
    In addition to time-zone-for-offset, you can use the time-zone-for-id and
-   default-time-zone functions and the utc Var to constgruct or get DateTimeZone
+   default-time-zone functions and the utc Var to construct or get DateTimeZone
    instances.
 
    The functions after? and before? determine the relative position of two
@@ -72,18 +72,25 @@
                  (date-time 1987))
      true
 
-   To find the amount of time encompased by an interval, use in-seconds and
+   To find the amount of time encompassed by an interval, use in-seconds and
    in-minutes:
 
      => (in-minutes (interval (date-time 1986 10 2) (date-time 1986 10 14)))
      17280
 
+   The overlap function can be used to get an Interval representing the
+   overlap between two intervals:
+
+     => (overlap (t/interval (t/date-time 1986) (t/date-time 1990))
+                             (t/interval (t/date-time 1987) (t/date-time 1991)))
+     #<Interval 1987-01-01T00:00:00.000Z/1990-01-01T00:00:00.000Z>
+
    Note that all functions in this namespace work with Joda objects or ints. If
    you need to print or parse date-times, see clj-time.format. If you need to
-   ceorce date-times to or from other types, see clj-time.coerce."
+   coerce date-times to or from other types, see clj-time.coerce."
   (:refer-clojure :exclude [extend second])
   (:import (org.joda.time ReadablePartial ReadableDateTime ReadableInstant ReadablePeriod DateTime
-                          DateMidnight YearMonth LocalDate DateTimeZone Period PeriodType Interval
+                          DateMidnight YearMonth LocalDate LocalTime DateTimeZone Period PeriodType Interval
                           Years Months Weeks Days Hours Minutes Seconds LocalDateTime MutableDateTime
                           DateTimeUtils)
            (org.joda.time.base BaseDateTime)))
@@ -102,12 +109,27 @@
   (sec [this]   "Return the second of minute component of the given date/time.")
   (second [this]   "Return the second of minute component of the given date/time.")
   (milli [this]   "Return the millisecond of second component of the given date/time.")
+  (equal? [this that] "Returns true if ReadableDateTime 'this' is strictly equal to date/time 'that'.")
   (after? [this that] "Returns true if ReadableDateTime 'this' is strictly after date/time 'that'.")
   (before? [this that] "Returns true if ReadableDateTime 'this' is strictly before date/time 'that'.")
   (plus- [this ^ReadablePeriod period]
     "Returns a new date/time corresponding to the given date/time moved forwards by the given Period(s).")
   (minus- [this ^ReadablePeriod period]
-    "Returns a new date/time corresponding to the given date/time moved backwards by the given Period(s)."))
+    "Returns a new date/time corresponding to the given date/time moved backwards by the given Period(s).")
+  (first-day-of-the-month- [this] "Returns the first day of the month")
+  (last-day-of-the-month- [this] "Returns the last day of the month")
+  (week-number-of-year [this] "Returs the number of weeks in the year"))
+
+(defprotocol InTimeUnitProtocol
+  "Interface for in-<time unit> functions"
+  (in-millis [this] "Return the time in milliseconds.")
+  (in-seconds [this] "Return the time in seconds.")
+  (in-minutes [this] "Return the time in minutes.")
+  (in-hours [this] "Return the time in hours.")
+  (in-days [this] "Return the time in days.")
+  (in-weeks [this] "Return the time in weeks")
+  (in-months [this] "Return the time in months")
+  (in-years [this] "Return the time in years"))
 
 (extend-protocol DateTimeProtocol
   org.joda.time.DateTime
@@ -123,10 +145,17 @@
     (.getSecondOfMinute this))
   (second [this] (.getSecondOfMinute this))
   (milli [this] (.getMillisOfSecond this))
+  (equal? [this ^ReadableInstant that] (.isEqual this that))
   (after? [this ^ReadableInstant that] (.isAfter this that))
   (before? [this ^ReadableInstant that] (.isBefore this that))
   (plus- [this ^ReadablePeriod period] (.plus this period))
   (minus- [this ^ReadablePeriod period] (.minus this period))
+  (first-day-of-the-month- [this]
+    (.. ^DateTime this dayOfMonth withMinimumValue))
+  (last-day-of-the-month- [this]
+     (.. ^DateTime this dayOfMonth withMaximumValue))
+  (week-number-of-year [this]
+    (.getWeekOfWeekyear this))
 
   org.joda.time.DateMidnight
   (year [this] (.getYear this))
@@ -141,10 +170,17 @@
     (.getSecondOfMinute this))
   (second [this] (.getSecondOfMinute this))
   (milli [this] (.getMillisOfSecond this))
+  (equal? [this ^ReadableInstant that] (.isEqual this that))
   (after? [this ^ReadableInstant that] (.isAfter this that))
   (before? [this ^ReadableInstant that] (.isBefore this that))
   (plus- [this ^ReadablePeriod period] (.plus this period))
   (minus- [this ^ReadablePeriod period] (.minus this period))
+  (first-day-of-the-month- [this]
+    (.. ^DateMidnight this dayOfMonth withMinimumValue))
+  (last-day-of-the-month- [this]
+     (.. ^DateMidnight this dayOfMonth withMaximumValue))
+  (week-number-of-year [this]
+    (.getWeekOfWeekyear this))
 
   org.joda.time.LocalDateTime
   (year [this] (.getYear this))
@@ -159,14 +195,22 @@
     (.getSecondOfMinute this))
   (second [this] (.getSecondOfMinute this))
   (milli [this] (.getMillisOfSecond this))
+  (equal? [this ^ReadablePartial that] (.isEqual this that))
   (after? [this ^ReadablePartial that] (.isAfter this that))
   (before? [this ^ReadablePartial that] (.isBefore this that))
   (plus- [this ^ReadablePeriod period] (.plus this period))
   (minus- [this ^ReadablePeriod period] (.minus this period))
+  (first-day-of-the-month- [this]
+    (.. ^LocalDateTime this dayOfMonth withMinimumValue))
+  (last-day-of-the-month- [this]
+     (.. ^LocalDateTime this dayOfMonth withMaximumValue))
+  (week-number-of-year [this]
+    (.getWeekOfWeekyear this))
 
   org.joda.time.YearMonth
   (year [this] (.getYear this))
   (month [this] (.getMonthOfYear this))
+  (equal? [this ^ReadablePartial that] (.isEqual this that))
   (after? [this ^ReadablePartial that] (.isAfter this that))
   (before? [this ^ReadablePartial that] (.isBefore this that))
   (plus- [this ^ReadablePeriod period] (.plus this period))
@@ -177,10 +221,29 @@
   (month [this] (.getMonthOfYear this))
   (day [this] (.getDayOfMonth this))
   (day-of-week [this] (.getDayOfWeek this))
+  (equal? [this ^ReadablePartial that] (.isEqual this that))
   (after? [this ^ReadablePartial that] (.isAfter this that))
   (before? [this ^ReadablePartial that] (.isBefore this that))
   (plus- [this ^ReadablePeriod period] (.plus this period))
-  (minus- [this ^ReadablePeriod period] (.minus this period)))
+  (minus- [this ^ReadablePeriod period] (.minus this period))
+  (first-day-of-the-month- [this]
+    (.. ^LocalDate this dayOfMonth withMinimumValue))
+  (last-day-of-the-month- [this]
+     (.. ^LocalDate this dayOfMonth withMaximumValue))
+  (week-number-of-year [this]
+    (.getWeekOfWeekyear this))
+
+  org.joda.time.LocalTime
+  (hour [this] (.getHourOfDay this))
+  (minute [this] (.getMinuteOfHour this))
+  (second [this] (.getSecondOfMinute this))
+  (milli [this] (.getMillisOfSecond this))
+  (equal? [this ^ReadablePartial that] (.isEqual this that))
+  (after? [this ^ReadablePartial that] (.isAfter this that))
+  (before? [this ^ReadablePartial that] (.isBefore this that))
+  (plus- [this ^ReadablePeriod period] (.plus this period))
+  (minus- [this ^ReadablePeriod period] (.minus this period))
+  )
 
 (def ^{:doc "DateTimeZone for UTC."}
       utc
@@ -191,6 +254,12 @@
   []
   (DateTime. ^DateTimeZone utc))
 
+(defn time-now
+  "Returns a LocalTime for the current instant without date or time zone
+  using ISOChronology in the current time zone."
+  []
+  (LocalTime. ))
+
 (defn today-at-midnight
   "Returns a DateMidnight for today at midnight in the UTC time zone."
   ([]
@@ -217,7 +286,7 @@
 
 (defn ^DateTime date-time
   "Constructs and returns a new DateTime in UTC.
-   Specify the year, month of year, day of month, hour of day, minute if hour,
+   Specify the year, month of year, day of month, hour of day, minute of hour,
    second of minute, and millisecond of second. Note that month and day are
    1-indexed while hour, second, minute, and millis are 0-indexed.
    Any number of least-significant components can be ommited, in which case
@@ -240,7 +309,7 @@
 
 (defn ^org.joda.time.LocalDateTime local-date-time
   "Constructs and returns a new LocalDateTime.
-   Specify the year, month of year, day of month, hour of day, minute if hour,
+   Specify the year, month of year, day of month, hour of day, minute of hour,
    second of minute, and millisecond of second. Note that month and day are
    1-indexed while hour, second, minute, and millis are 0-indexed.
    Any number of least-significant components can be ommited, in which case
@@ -276,6 +345,21 @@
   [^Integer year ^Integer month ^Integer day]
   (LocalDate. year month day))
 
+(defn ^org.joda.time.LocalTime local-time
+  "Constructs and returns a new LocalTime.
+   Specify the hour of day, minute of hour, second of minute, and millisecond of second.
+   Any number of least-significant components can be ommited, in which case
+   they will default to 1 or 0 as appropriate."
+  ([hour]
+   (local-time hour 0 0 0))
+  ([hour minute]
+   (local-time hour minute 0 0))
+  ([hour minute second]
+   (local-time hour minute second 0))
+  ([^Integer hour ^Integer minute ^Integer second ^Integer millis]
+   (LocalTime. hour minute second millis))
+  )
+
 (defn ^org.joda.time.LocalDate today
   "Constructs and returns a new LocalDate representing today's date.
    LocalDate objects do not deal with timezones at all."
@@ -296,6 +380,11 @@
   [^String id]
   (DateTimeZone/forID id))
 
+(defn available-ids
+  "Returns a set of available IDs for use with time-zone-for-id."
+  []
+  (DateTimeZone/getAvailableIDs))
+
 (defn default-time-zone
   "Returns the default DateTimeZone for the current environment."
   []
@@ -373,6 +462,52 @@
   ([^Integer n]
      (Seconds/seconds n)))
 
+(extend-protocol InTimeUnitProtocol
+  org.joda.time.Interval 
+  (in-millis [this] (.toDurationMillis this))
+  (in-seconds [this] (.getSeconds (.toPeriod this (seconds))))
+  (in-minutes [this] (.getMinutes (.toPeriod this (minutes))))
+  (in-hours [this] (.getHours (.toPeriod this (hours))))
+  (in-days [this] (.getDays (.toPeriod this (days))))
+  (in-weeks [this] (.getWeeks (.toPeriod this (weeks))))
+  (in-months [this] (.getMonths (.toPeriod this (months))))
+  (in-years [this] (.getYears (.toPeriod this (years))))
+  org.joda.time.ReadablePeriod
+  (in-millis [this] (-> this .toPeriod .toStandardDuration .getMillis))
+  (in-seconds [this] (-> this .toPeriod .toStandardSeconds .getSeconds))
+  (in-minutes [this] (-> this .toPeriod .toStandardMinutes .getMinutes))
+  (in-hours [this] (-> this .toPeriod .toStandardHours .getHours))
+  (in-days [this] (-> this .toPeriod .toStandardDays .getDays))
+  (in-weeks [this] (-> this .toPeriod .toStandardWeeks .getWeeks))
+  (in-months [this] 
+    (condp instance? this
+      org.joda.time.Months (.getMonths ^org.joda.time.Months this)
+      org.joda.time.Years (* 12 (.getYears ^org.joda.time.Years this))
+      (throw
+        (UnsupportedOperationException.
+          "Cannot convert to Months because months vary in length."))))
+  (in-years [this]
+    (condp instance? this
+      org.joda.time.Months (int (/ (.getMonths ^org.joda.time.Months this) 12))
+      org.joda.time.Years (.getYears ^org.joda.time.Years this)
+      (throw
+        (UnsupportedOperationException.
+          "Cannot convert to Years because years vary in length.")))))
+
+(defn in-msecs
+  "DEPRECATED: Returns the number of milliseconds in the given Interval."
+  {:deprecated "0.6.0"}
+  [^Interval in]
+  (deprecated "in-msecs has been deprecated in favor of in-millis")
+  (in-millis in))
+ 
+(defn in-secs
+  "DEPRECATED: Returns the number of standard seconds in the given Interval."
+  {:deprecated "0.6.0"}
+  [^Interval in]
+  (deprecated "in-secs has been deprecated in favor of in-seconds")
+  (in-seconds in))
+ 
 (defn secs
   "DEPRECATED"
   {:deprecated "0.6.0"}
@@ -413,6 +548,11 @@
   [^Period period]
   (minus (now) period))
 
+(defn yesterday
+  "Returns a DateTime for yesterday relative to now"
+  []
+  (-> 1 days ago))
+
 (defn from-now
   "Returns a DateTime a supplied period after the present.
   e.g. (-> 30 minutes from-now)"
@@ -457,59 +597,6 @@
   [^Interval in & by]
   (.withEnd in (apply plus (end in) by)))
 
-(defn in-millis
-  "Returns the number of milliseconds in the given Interval."
-  [^Interval in]
-  (.toDurationMillis in))
-
-(defn in-msecs
-  "DEPRECATED: Returns the number of milliseconds in the given Interval."
-  {:deprecated "0.6.0"}
-  [^Interval in]
-  (deprecated "in-msecs has been deprecated in favor of in-millis")
-  (in-millis in))
-
-(defn in-seconds
-  "Returns the number of standard seconds in the given Interval."
-  [^Interval in]
-  (.getSeconds (.toPeriod in (seconds))))
-
-(defn in-secs
-  "DEPRECATED: Returns the number of standard seconds in the given Interval."
-  {:deprecated "0.6.0"}
-  [^Interval in]
-  (deprecated "in-secs has been deprecated in favor of in-seconds")
-  (in-seconds in))
-
-(defn in-minutes
-  "Returns the number of standard minutes in the given Interval."
-  [^Interval in]
-  (.getMinutes (.toPeriod in (minutes))))
-
-(defn in-hours
-  "Returns the number of standard hours in the given Interval."
-  [^Interval in]
-  (.getHours (.toPeriod in (hours))))
-
-(defn in-days
-  "Returns the number of standard days in the given Interval."
-  [^Interval in]
-  (.getDays (.toPeriod in (days))))
-
-(defn in-weeks
-  "Returns the number of standard weeks in the given Interval."
-  [^Interval in]
-  (.getWeeks (.toPeriod in (weeks))))
-
-(defn in-months
-  "Returns the number of standard months in the given Interval."
-  [^Interval in]
-  (.getMonths (.toPeriod in (months))))
-
-(defn in-years
-  "Returns the number of standard years in the given Interval."
-  [^Interval in]
-  (.getYears (.toPeriod in (years))))
 
 (defn within?
   "With 2 arguments: Returns true if the given Interval contains the given
@@ -521,8 +608,8 @@
   ([^Interval i ^ReadableDateTime dt]
      (.contains i dt))
   ([^ReadablePartial start ^ReadablePartial end ^ReadablePartial test]
-     (or (= start test)
-         (= end test)
+     (or (equal? start test)
+         (equal? end test)
          (and (before? start test) (after? end test)))))
 
 (defn overlaps?
@@ -536,7 +623,22 @@
     ^ReadablePartial start-b ^ReadablePartial end-b]
      (or (and (before? start-b end-a) (after? end-b start-a))
          (and (after? end-b start-a) (before? start-b end-a))
-         (or (= start-a end-b) (= start-b end-a)))))
+         (or (equal? start-a end-b) (equal? start-b end-a)))))
+
+(defn overlap
+  "Returns an Interval representing the overlap of the specified Intervals.
+   Returns nil if the Intervals do not overlap.
+   The first argument must not be nil.
+   If the second argument is nil then the overlap of the first argument
+   and a zero duration interval with both start and end times equal to the
+   current time is returned."
+  [^Interval i-a ^Interval i-b]
+     ;; joda-time AbstractInterval.overlaps:
+     ;;    null argument means a zero length interval 'now'.
+     (cond (nil? i-b) (let [n (now)] (overlap i-a (interval n n)))
+           (.overlaps i-a i-b) (interval (latest (start i-a) (start i-b))
+                                         (earliest (end i-a) (end i-b)))
+           :else nil))
 
 (defn abuts?
   "Returns true if Interval i-a abuts i-b, i.e. then end of i-a is exactly the
@@ -590,25 +692,23 @@
   [d]
   (in-minutes (interval d (now))))
 
-
-(defn ^DateTime last-day-of-the-month
-  ([^DateTime dt]
-     (last-day-of-the-month (.getYear dt) (.getMonthOfYear dt)))
+(defn first-day-of-the-month
   ([^long year ^long month]
-     (-> ^DateTime (date-time year month) .dayOfMonth .withMaximumValue)))
+     (first-day-of-the-month- (date-time year month)))
+  ([dt]
+     (first-day-of-the-month- dt)))
+
+(defn last-day-of-the-month
+  ([^long year ^long month]
+     (last-day-of-the-month- (date-time year month)))
+  ([dt]
+     (last-day-of-the-month- dt)))
 
 (defn number-of-days-in-the-month
   (^long [^DateTime dt]
-         (number-of-days-in-the-month (.getYear dt) (.getMonthOfYear dt)))
+         (day (last-day-of-the-month- dt)))
   (^long [^long year ^long month]
-         (.getDayOfMonth ^DateTime (last-day-of-the-month year month))))
-
-(defn ^DateTime first-day-of-the-month
-  ([^DateTime dt]
-     (first-day-of-the-month (.getYear dt) (.getMonthOfYear dt)))
-  ([^long year ^long month]
-     (-> ^DateTime (date-time year month) .dayOfMonth .withMinimumValue)))
-
+         (day (last-day-of-the-month- (date-time year month)))))
 
 (defn ^DateTime today-at
   ([^long hours ^long minutes ^long seconds ^long millis]
