@@ -28,6 +28,13 @@
           profiler          (get-profiler)]
       (core/verify-config-found! config)
       (log/info "Initializing the JRuby service")
+      (if (:use-legacy-auth-conf config)
+        (log/warn "The 'jruby-puppet.use-legacy-auth-conf' setting is set to"
+                  "'true'.  Support for the legacy Puppet auth.conf file is"
+                  "deprecated and will be removed in a future release.  Change"
+                  "this setting to 'false' and migrate your authorization rule"
+                  "definitions in the /etc/puppetlabs/puppet/auth.conf file to"
+                  "the /etc/puppetlabs/puppetserver/conf.d/auth.conf file."))
       (core/add-facter-jar-to-system-classloader (:ruby-load-path config))
       (let [pool-context (core/create-pool-context config profiler agent-shutdown-fn)]
         (jruby-agents/send-prime-pool! pool-context)
@@ -105,3 +112,18 @@
            ~@body
            (finally
              (jruby/return-instance ~jruby-service pool-instance# ~reason)))))))
+
+(defmacro with-lock
+  "Acquires a lock on the pool, executes the body, and releases the lock."
+  [jruby-service reason & body]
+  `(let [context# (-> ~jruby-service
+                      tk-services/service-context)
+         pool# (-> context#
+                   :pool-context
+                   core/get-pool)
+         event-callbacks# (:event-callbacks context#)]
+     (core/lock-pool pool# ~reason @event-callbacks#)
+     (try
+      ~@body
+      (finally
+        (core/unlock-pool pool# ~reason @event-callbacks#)))))
