@@ -9,12 +9,9 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Constants
 
-(def puppet-API-versions
+(def puppet-API-version
   "v3")
 
-(def puppet-ca-API-versions
-  "v1")
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Routing
 
@@ -26,8 +23,6 @@
   (comidi/routes
     (comidi/GET ["/node/" [#".*" :rest]] request
                    (request-handler request))
-    (comidi/GET ["/facts/" [#".*" :rest]] request
-                   (request-handler request))
     (comidi/GET ["/file_content/" [#".*" :rest]] request
                    (request-handler request))
     (comidi/GET ["/file_metadatas/" [#".*" :rest]] request
@@ -51,15 +46,11 @@
                    (request-handler request))
     (comidi/GET ["/resource_types/" [#".*" :rest]] request
                    (request-handler request))
+    (comidi/GET ["/environment/" [#".*" :rest]] request
+                   (request-handler request))
     (comidi/GET "/environments" request
                    (request-handler request))
     (comidi/GET ["/status/" [#".*" :rest]] request
-                   (request-handler request))
-
-    ;; TODO: when we get rid of the legacy dashboard after 3.4, we should remove
-    ;; this endpoint as well.  It makes more sense for this type of query to be
-    ;; directed to PuppetDB.
-    (comidi/GET ["/facts_search/" [#".*" :rest]] request
                    (request-handler request))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -148,3 +139,14 @@
     :else
     (throw (IllegalArgumentException.
              (str "Route not found for service " master-ns)))))
+
+(schema/defn ^:always-validate
+  get-wrapped-handler :- IFn
+  [route-handler :- IFn
+   authorization-fn :- IFn
+   use-legacy-auth-conf :- schema/Bool
+   puppet-version :- schema/Str]
+  (let [handler-maybe-with-authorization (if use-legacy-auth-conf
+                                           route-handler
+                                           (authorization-fn route-handler))]
+    (wrap-middleware handler-maybe-with-authorization puppet-version)))
